#*******************************************************************************
#**  This file is part of Dux Rei Nummariae.                                 ***
#**                                                                          ***
#**  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
#**  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
#**                                                                          ***
#**  This program is free software: you can redistribute it and/or modify it ***
#**  under the terms of the GNU Affero General Public License as published   ***
#**  by the Free Software Foundation, either version 3 of the License, or    ***
#**  (at your option) any later version.                                     ***
#**                                                                          ***
#**  This program is distributed in the hope that it will be useful, but     ***
#**  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
#**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
#**  See the GNU Affero General Public License for more details.             ***
#**                                                                          ***
#**  You should have received a copy of the GNU Affero General Public License***
#**  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
#*******************************************************************************

%define so_number	0

Name:			duxreinummariae
Version:		%{getenv:BINARY_VERSION}
Release:		1%distsuffix%dist
Summary:		Dux Rei Nummariae (Budget Navigator)
Source0:		https://packaging.schneiderman.me/svn/sources/%{name}-%{version}.tar.gz
URL:			http://www.budgetnav.com

Group:			Office
License:		AGPLv3+

BuildRequires:		pecunia-devel = 0.5.1
Requires:		pecunia = 0.5.1


%description
A budgeting application that is simple to use without sacrificing any
loss of predictive powers.


%package devel
Summary: Development headers
Requires: %{name}
BuildArch: noarch
%description devel
Provides the headers needed for development.

%prep
%setup -q -a 0

%build
if [ -z "%{getenv:CURRENCY_API_KEY}" ]
then
	echo "Missing Currency API key."
	exit 1
fi
%cmake \
	-DLIB_INSTALL_DIR=%{_libdir} \
	-DDuxReiNummariae_OPEN_EXCHANGE_RATES_API_KEY="%{getenv:CURRENCY_API_KEY}"

%if 0%{?cmake_build:1}
	%cmake_build
%else
	cd %_build
	%make_build
%endif

%install
%if 0%{?cmake_install:1}
	%cmake_install
%else
	cd %_build
	%make_install
%endif

%files
%doc ChangeLog CREDITS
%{_bindir}/drn-desktop
%{_bindir}/drn-desktop-%{version}
%{_libdir}/libaccounting.so
%{_libdir}/libaccounting.so.%{so_number}
%{_libdir}/libaccounting.so.%{version}
%{_libdir}/libadviser.so
%{_libdir}/libadviser.so.%{so_number}
%{_libdir}/libadviser.so.%{version}
%{_libdir}/libbanking.so
%{_libdir}/libbanking.so.%{so_number}
%{_libdir}/libbanking.so.%{version}
%{_libdir}/libbudgeting.so
%{_libdir}/libbudgeting.so.%{so_number}
%{_libdir}/libbudgeting.so.%{version}
%{_libdir}/libconversion.so
%{_libdir}/libconversion.so.%{so_number}
%{_libdir}/libconversion.so.%{version}
%{_libdir}/libdesktop-ui.so
%{_libdir}/libdesktop-ui.so.%{so_number}
%{_libdir}/libdesktop-ui.so.%{version}
%{_libdir}/libfile-storage.so
%{_libdir}/libfile-storage.so.%{so_number}
%{_libdir}/libfile-storage.so.%{version}
%{_libdir}/libfoundation.so
%{_libdir}/libfoundation.so.%{so_number}
%{_libdir}/libfoundation.so.%{version}
%{_libdir}/libnavigation.so
%{_libdir}/libnavigation.so.%{so_number}
%{_libdir}/libnavigation.so.%{version}
%{_libdir}/libnetwork.so
%{_libdir}/libnetwork.so.%{so_number}
%{_libdir}/libnetwork.so.%{version}

###
### Devel
###
%files devel
%{_includedir}/drn/accounting/Account.h
%{_includedir}/drn/accounting/AccountCode.h
%{_includedir}/drn/accounting/AccountNumber.h
%{_includedir}/drn/accounting/AccountNumberGenerator.h
%{_includedir}/drn/accounting/AccountTypes.h
%{_includedir}/drn/accounting/AccountingErrors.h
%{_includedir}/drn/accounting/LedgerBooks.h
%{_includedir}/drn/accounting/Transaction.h
%{_includedir}/drn/accounting/accounting_export.h
%{_includedir}/drn/adviser/BudgetHealth.h
%{_includedir}/drn/adviser/adviser_export.h
%{_includedir}/drn/banking/Bank.h
%{_includedir}/drn/banking/BankAccount.h
%{_includedir}/drn/banking/BankAccountType.h
%{_includedir}/drn/banking/BankMap.h
%{_includedir}/drn/banking/BankName.h
%{_includedir}/drn/banking/BankingErrors.h
%{_includedir}/drn/banking/Reconciliation.h
%{_includedir}/drn/banking/banking_export.h
%{_includedir}/drn/budgeting/Algorithm.h
%{_includedir}/drn/budgeting/Bill.h
%{_includedir}/drn/budgeting/BillMap.hpp
%{_includedir}/drn/budgeting/Budget.h
%{_includedir}/drn/budgeting/BudgetItemDue.h
%{_includedir}/drn/budgeting/BudgetItemIdentifier.h
%{_includedir}/drn/budgeting/BudgetItemTypes.h
%{_includedir}/drn/budgeting/BudgetSource.h
%{_includedir}/drn/budgeting/BudgetedMoney.h
%{_includedir}/drn/budgeting/BudgetedMoneyMap.hpp
%{_includedir}/drn/budgeting/BudgetingErrors.h
%{_includedir}/drn/budgeting/Constants.hpp
%{_includedir}/drn/budgeting/Debt.h
%{_includedir}/drn/budgeting/DebtMap.h
%{_includedir}/drn/budgeting/EventFrequency.h
%{_includedir}/drn/budgeting/Goal.h
%{_includedir}/drn/budgeting/GoalMap.h
%{_includedir}/drn/budgeting/Nontrack.h
%{_includedir}/drn/budgeting/NontrackMap.hpp
%{_includedir}/drn/budgeting/Wage.h
%{_includedir}/drn/budgeting/WageMap.hpp
%{_includedir}/drn/budgeting/budgeting_export.h
%{_includedir}/drn/config.h
%{_includedir}/drn/conversion/Constants.hpp
%{_includedir}/drn/conversion/ConversionErrors.h
%{_includedir}/drn/conversion/ConversionMap.h
%{_includedir}/drn/conversion/CurrencyConverter.h
%{_includedir}/drn/conversion/CurrencySource.hpp
%{_includedir}/drn/conversion/OpenExchangeRatesSource.h
%{_includedir}/drn/conversion/conversion_export.h
%{_includedir}/drn/desktop-ui/CurrencyCodeWidget.h
%{_includedir}/drn/desktop-ui/DesktopUiErrors.h
%{_includedir}/drn/desktop-ui/EventFrequencyEntryWidget.h
%{_includedir}/drn/desktop-ui/MainWindow.h
%{_includedir}/drn/desktop-ui/MoneyEntryWidget.h
%{_includedir}/drn/desktop-ui/VerticalLabel.h
%{_includedir}/drn/desktop-ui/desktop-ui_export.h
%{_includedir}/drn/drn-desktop/Application.h
%{_includedir}/drn/file-storage/BudgetFile.h
%{_includedir}/drn/file-storage/Constants.hpp
%{_includedir}/drn/file-storage/ExchangeRatesFile.h
%{_includedir}/drn/file-storage/FileStorageErrors.h
%{_includedir}/drn/file-storage/file-storage_export.h
%{_includedir}/drn/foundation/CalendarTime.hpp
%{_includedir}/drn/foundation/EnumMacros.hpp
%{_includedir}/drn/foundation/Error.h
%{_includedir}/drn/foundation/ObserverPtr.hpp
%{_includedir}/drn/foundation/Optional.hpp
%{_includedir}/drn/foundation/PresentationText.h
%{_includedir}/drn/foundation/QtMemory.hpp
%{_includedir}/drn/foundation/ScopeExecutor.h
%{_includedir}/drn/foundation/Switch.h
%{_includedir}/drn/foundation/TypeIndexMap.hpp
%{_includedir}/drn/foundation/foundation_export.h
%{_includedir}/drn/navigation/BankAccountSurvey.h
%{_includedir}/drn/navigation/BudgetBankAccount.h
%{_includedir}/drn/navigation/Navigator.h
%{_includedir}/drn/navigation/UserPreferences.h
%{_includedir}/drn/navigation/navigation_export.h
%{_includedir}/drn/network/AccessManager.hpp
%{_includedir}/drn/network/NetworkErrors.h
%{_includedir}/drn/network/QtAccessManager.h
%{_includedir}/drn/network/network_export.h


%changelog
* Mon Dec 27 2021 John Schneiderman <JohnMS@member.fsf.org> 0.4.0-1jms
- Initial packaging.
