/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021                              ***
***  John Schneiderman <Licensing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include <QString>
using ::QString;
#include <QTest>
// using QTEST_MAIN

#include <foundation/Error.h>
using drn::foundation::Error;
#include <budgeting/BudgetSource.h>
using drn::budgeting::BudgetSource;


namespace drn
{
namespace budgeting
{
namespace tests
{

struct BudgetSourceUT : QObject
{
	Q_OBJECT

private slots:
	//{ Constructor Tests

	void constructor_Default_ShouldSet()
	{
		const QString name{};
		const BudgetSource bs{};

		QCOMPARE(bs.name(), name);
	}
	void constructor_ValidParams_ShouldSet()
	{
		const QString name{"McDonald's"};
		const BudgetSource bs{name};

		QCOMPARE(bs.name(), name);
	}
	void constructor_ValidNameWithSpaces_ShouldSet()
	{
		const QString name{"McDonald's"};
		const BudgetSource bs{"   McDonald's   "};

		QCOMPARE(bs.name(), name);
	}
	void constructor_InvalidName_ShouldThrow()
	{
		QVERIFY_EXCEPTION_THROWN(BudgetSource("   "), Error);
	}

	//}

	//{ setName Tests

	void setSource_ValidParams_ShouldSet()
	{
		const QString name{"McDonald's"};
		BudgetSource bs{name};
		bs.setName("Burger King");
		QCOMPARE(bs.name(), QStringLiteral("Burger King"));
	}
	void setSource_ValidNameWithSpaces_ShouldSet()
	{
		const QString name{"McDonald's"};
		BudgetSource bs{name};
		bs.setName("   Burger King   ");
		QCOMPARE(bs.name(), QStringLiteral("Burger King"));
	}
	void setSource_EmptyName_ShouldThrow()
	{
		const QString name{"McDonald's"};
		BudgetSource bs{name};
		QVERIFY_EXCEPTION_THROWN(bs.setName("   "), Error);
	}

	//}
};

}}}

QTEST_MAIN(drn::budgeting::tests::BudgetSourceUT)
#include "BudgetSource-unit-tests.moc"
